@@ -22,7 +22,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#define AUD_PLUGIN_GLIB_ONLY
 #include <libaudcore/audstrings.h>
 #include <libaudcore/drct.h>
 #include <libaudcore/hook.h>
@@ -43,6 +42,9 @@
 #include "ui_statusbar.h"
 #include "playlist_util.h"
 
+#include "../ui-common/menu-ops.cc"
+#include "../ui-common/menu-ops-gtk.cc"
+
 static const char * const gtkui_defaults[] = {
     "infoarea_show_vis", "TRUE",
     "infoarea_visible", "TRUE",
@@ -61,6 +63,8 @@ static const char * const gtkui_defaults[] = {
 
     "player_x", "-1000",
     "player_y", "-1000",
+    "player_width", "768",
+    "player_height", "480",
 
     nullptr
 };
@@ -72,7 +76,8 @@ public:
         N_("GTK Interface"),
         PACKAGE,
         nullptr,
-        & gtkui_prefs
+        & gtkui_prefs,
+        PluginGLibOnly
     };
 
     constexpr GtkUI () : IfacePlugin (info) {}
@@ -106,6 +111,9 @@ public:
         { audgui_plugin_menu_add (id, func, name, icon); }
     void plugin_menu_remove (AudMenuID id, void func ())
         { audgui_plugin_menu_remove (id, func); }
+
+    void startup_notify (const char * id)
+        { gdk_notify_startup_complete_with_id (id); }
 };
 
 EXPORT GtkUI aud_plugin_instance;
@@ -137,22 +145,16 @@ static void save_window_size ()
 
     aud_set_int ("gtkui", "player_x", x);
     aud_set_int ("gtkui", "player_y", y);
-    aud_set_int ("gtkui", "player_width", w);
-    aud_set_int ("gtkui", "player_height", h);
+    aud_set_int ("gtkui", "player_width", audgui_to_portable_dpi (w));
+    aud_set_int ("gtkui", "player_height", audgui_to_portable_dpi (h));
 }
 
 static void restore_window_size ()
 {
-    int dpi = audgui_get_dpi ();
     int x = aud_get_int ("gtkui", "player_x");
     int y = aud_get_int ("gtkui", "player_y");
-    int w = aud_get_int ("gtkui", "player_width");
-    int h = aud_get_int ("gtkui", "player_height");
-
-    if (w < 1)
-        w = 8 * dpi;
-    if (h < 1)
-        h = 5 * dpi;
+    int w = audgui_to_native_dpi (aud_get_int ("gtkui", "player_width"));
+    int h = audgui_to_native_dpi (aud_get_int ("gtkui", "player_height"));
 
     gtk_window_set_default_size ((GtkWindow *) window, w, h);
 
@@ -214,19 +216,23 @@ static void title_change (void * = nullptr)
 {
     delayed_title_change.stop ();
 
+    StringBuf title;
+
     if (aud_drct_get_playing ())
     {
         if (aud_drct_get_ready ())
-        {
-            String title = aud_drct_get_title ();
-            gtk_window_set_title ((GtkWindow *) window,
-             str_printf (_("%s - Audacious"), (const char *) title));
-        }
+            title.steal (str_printf (_("%s - Audacious"), (const char *) aud_drct_get_title ()));
         else
-            gtk_window_set_title ((GtkWindow *) window, _("Buffering ..."));
+            title.steal (str_copy (_("Buffering ...")));
     }
     else
-        gtk_window_set_title ((GtkWindow *) window, _("Audacious"));
+        title.steal (str_copy (_("Audacious")));
+
+    int instance = aud_get_instance ();
+    if (instance != 1)
+        title.combine (str_printf (" (%d)", instance));
+
+    gtk_window_set_title ((GtkWindow *) window, title);
 }
 
 void GtkUI::show (bool show)
@@ -334,11 +340,20 @@ static gboolean ui_slider_change_value_cb (GtkRange * range,
     return false;
 }
 
+/* return an object property if it exists, otherwise false */
+static bool get_boolean_prop (void * obj, const char * prop)
+{
+    gboolean value = false;
+    if (g_object_class_find_property (G_OBJECT_GET_CLASS (obj), prop))
+        g_object_get (obj, prop, & value, nullptr);
+
+    return value;
+}
+
 static gboolean ui_slider_button_press_cb (GtkWidget * widget, GdkEventButton * event)
 {
-    gboolean primary_warps = false;
-    g_object_get (gtk_widget_get_settings (widget),
-     "gtk-primary-button-warps-slider", & primary_warps, nullptr);
+    bool primary_warps = get_boolean_prop (gtk_widget_get_settings (widget),
+     "gtk-primary-button-warps-slider");
 
     if (event->button == 1 && ! primary_warps)
         event->button = 2;
@@ -349,9 +364,8 @@ static gboolean ui_slider_button_press_cb (GtkWidget * widget, GdkEventButton *
 
 static gboolean ui_slider_button_release_cb (GtkWidget * widget, GdkEventButton * event)
 {
-    gboolean primary_warps = false;
-    g_object_get (gtk_widget_get_settings (widget),
-     "gtk-primary-button-warps-slider", & primary_warps, nullptr);
+    bool primary_warps = get_boolean_prop (gtk_widget_get_settings (widget),
+     "gtk-primary-button-warps-slider");
 
     if (event->button == 1 && ! primary_warps)
         event->button = 2;
@@ -538,8 +552,7 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event)
         {
           case GDK_KEY_ISO_Left_Tab:
           case GDK_KEY_Tab:
-            aud_playlist_set_active ((aud_playlist_get_active () + 1) %
-             aud_playlist_count ());
+            pl_next ();
             break;
 
           default:
@@ -551,8 +564,7 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event)
         {
           case GDK_KEY_ISO_Left_Tab:
           case GDK_KEY_Tab:
-            aud_playlist_set_active (aud_playlist_get_active () ?
-             aud_playlist_get_active () - 1 : aud_playlist_count () - 1);
+            pl_prev ();
             break;
           default:
             return false;
@@ -590,7 +602,7 @@ static gboolean playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event)
             ui_playlist_notebook_position (aud::to_ptr (aud_playlist_get_active ()), nullptr);
             return true;
         case GDK_KEY_Delete:
-            playlist_delete_selected ();
+            pl_remove_selected ();
             return true;
         case GDK_KEY_Menu:
             popup_menu_rclick (0, event->time);
@@ -602,16 +614,16 @@ static gboolean playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event)
         switch (event->keyval)
         {
         case 'x':
-            playlist_cut ();
+            pl_cut ();
             return true;
         case 'c':
-            playlist_copy ();
+            pl_copy ();
             return true;
         case 'v':
-            playlist_paste ();
+            pl_paste ();
             return true;
         case 'a':
-            aud_playlist_select_all (aud_playlist_get_active (), true);
+            pl_select_all ();
             return true;
         }
 
