@@ -30,7 +30,6 @@
 #include <QPushButton>
 #include <QTreeView>
 
-#define AUD_PLUGIN_QT_ONLY
 #include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudcore/i18n.h>
@@ -50,7 +49,10 @@ class SearchToolQt : public GeneralPlugin
 public:
     static constexpr PluginInfo info = {
         N_("Search Tool"),
-        PACKAGE
+        PACKAGE,
+        nullptr, // about
+        nullptr, // prefs
+        PluginQtOnly
     };
 
     constexpr SearchToolQt () : GeneralPlugin (info, false) {}
@@ -225,17 +227,20 @@ static int get_playlist (bool require_added, bool require_scanned)
     return list;
 }
 
-static String get_path ()
+static String get_uri ()
 {
+    auto to_uri = [] (const char * path)
+        { return String (filename_to_uri (path)); };
+
     String path1 = aud_get_str ("search-tool", "path");
-    if (g_file_test (path1, G_FILE_TEST_EXISTS))
-        return path1;
+    if (path1[0])
+        return strstr (path1, "://") ? path1 : to_uri (path1);
 
     StringBuf path2 = filename_build ({g_get_home_dir (), "Music"});
     if (g_file_test (path2, G_FILE_TEST_EXISTS))
-        return String (path2);
+        return to_uri (path2);
 
-    return String (g_get_home_dir ());
+    return to_uri (g_get_home_dir ());
 }
 
 static void destroy_database ()
@@ -254,7 +259,7 @@ static void create_database (int list)
 
     for (int e = 0; e < entries; e ++)
     {
-        Tuple tuple = aud_playlist_entry_get_tuple (list, e, Playlist::Nothing);
+        Tuple tuple = aud_playlist_entry_get_tuple (list, e, Playlist::NoWait);
 
         aud::array<SearchField, String> fields;
         fields[SearchField::Genre] = tuple.get_str (Tuple::Genre);
@@ -317,7 +322,7 @@ static void search_cb (const Key & key, Item & item, void * _state)
     state->mask = oldmask;
 }
 
-static int item_compare (const Item * const & a, const Item * const & b, void *)
+static int item_compare (const Item * const & a, const Item * const & b)
 {
     if (a->field < b->field)
         return -1;
@@ -329,19 +334,19 @@ static int item_compare (const Item * const & a, const Item * const & b, void *)
         return val;
 
     if (a->parent)
-        return b->parent ? item_compare (a->parent, b->parent, nullptr) : 1;
+        return b->parent ? item_compare (a->parent, b->parent) : 1;
     else
         return b->parent ? -1 : 0;
 }
 
-static int item_compare_pass1 (const Item * const & a, const Item * const & b, void *)
+static int item_compare_pass1 (const Item * const & a, const Item * const & b)
 {
     if (a->matches.len () > b->matches.len ())
         return -1;
     if (a->matches.len () < b->matches.len ())
         return 1;
 
-    return item_compare (a, b, nullptr);
+    return item_compare (a, b);
 }
 
 static void do_search ()
@@ -362,7 +367,7 @@ static void do_search ()
     items = std::move (state.items);
 
     /* first sort by number of songs per item */
-    items.sort (item_compare_pass1, nullptr);
+    items.sort (item_compare_pass1);
 
     /* limit to items with most songs */
     if (items.len () > MAX_RESULTS)
@@ -372,18 +377,29 @@ static void do_search ()
     }
 
     /* sort by item type, then item name */
-    items.sort (item_compare, nullptr);
+    items.sort (item_compare);
 }
 
 static bool filter_cb (const char * filename, void * unused)
 {
+    bool add = false;
     tiny_lock (& adding_lock);
-    bool add = adding && ! added_table.lookup (String (filename));
+
+    if (adding)
+    {
+        bool * added = added_table.lookup (String (filename));
+
+        if ((add = ! added))
+            added_table.add (String (filename), true);
+        else
+            (* added) = true;
+    }
+
     tiny_unlock (& adding_lock);
     return add;
 }
 
-static void begin_add (const char * path)
+static void begin_add (const char * uri)
 {
     if (adding)
         return;
@@ -393,13 +409,9 @@ static void begin_add (const char * path)
     if (list < 0)
         list = create_playlist ();
 
-    aud_set_str ("search-tool", "path", path);
-
-    StringBuf uri = filename_to_uri (path);
-    g_return_if_fail (uri);
-
-    if (! g_str_has_suffix (uri, "/"))
-        uri.insert (-1, "/");
+    /* if possible, store local path for compatibility with older versions */
+    StringBuf path = uri_to_filename (uri);
+    aud_set_str ("search-tool", "path", path ? path : uri);
 
     added_table.clear ();
 
@@ -409,17 +421,16 @@ static void begin_add (const char * path)
     {
         String filename = aud_playlist_entry_get_filename (list, entry);
 
-        if (g_str_has_prefix (filename, uri) && ! added_table.lookup (filename))
+        if (! added_table.lookup (filename))
         {
             aud_playlist_entry_set_selected (list, entry, false);
-            added_table.add (filename, true);
+            added_table.add (filename, false);
         }
         else
             aud_playlist_entry_set_selected (list, entry, true);
     }
 
     aud_playlist_delete_selected (list);
-    aud_playlist_remove_failed (list);
 
     tiny_lock (& adding_lock);
     adding = true;
@@ -518,7 +529,24 @@ static void add_complete_cb (void * unused, void * unused2)
         adding = false;
         tiny_unlock (& adding_lock);
 
+        int entries = aud_playlist_entry_count (list);
+
+        for (int entry = 0; entry < entries; entry ++)
+        {
+            String filename = aud_playlist_entry_get_filename (list, entry);
+            bool * added = added_table.lookup (filename);
+
+            aud_playlist_entry_set_selected (list, entry, ! added || ! (* added));
+        }
+
         added_table.clear ();
+
+        /* don't clear the playlist if nothing was added */
+        if (aud_playlist_selected_count (list) < aud_playlist_entry_count (list))
+            aud_playlist_delete_selected (list);
+        else
+            aud_playlist_select_all (list, false);
+
         aud_playlist_sort_by_scheme (list, Playlist::Path);
     }
 
@@ -606,7 +634,8 @@ static void do_add (bool play, bool set_title)
         {
             add.append (
                 aud_playlist_entry_get_filename (list, entry),
-                aud_playlist_entry_get_tuple (list, entry, Playlist::Nothing)
+                aud_playlist_entry_get_tuple (list, entry, Playlist::NoWait),
+                aud_playlist_entry_get_decoder (list, entry, Playlist::NoWait)
             );
         }
 
@@ -624,14 +653,7 @@ static void do_add (bool play, bool set_title)
 
 static void action_play ()
 {
-    int list = aud_playlist_get_temporary ();
-    aud_playlist_set_active (list);
-
-    if (aud_get_bool (nullptr, "clear_playlist"))
-        aud_playlist_entry_delete (list, 0, aud_playlist_entry_count (list));
-    else
-        aud_playlist_queue_delete (list, 0, aud_playlist_queue_count (list));
-
+    aud_playlist_set_active (aud_playlist_get_temporary ());
     do_add (true, false);
 }
 
@@ -734,9 +756,9 @@ void * SearchToolQt::get_qt_widget ()
     button->setFlat (true);
     hbox->addWidget (button);
 
-    char * utf8 = g_filename_to_utf8 (get_path (), -1, nullptr, nullptr, nullptr);
-    chooser->setText (utf8);
-    g_free (utf8);
+    String uri = get_uri ();
+    StringBuf path = uri_to_filename (uri, false);
+    chooser->setText (path ? path : uri);
 
     search_init ();
 
@@ -751,19 +773,20 @@ void * SearchToolQt::get_qt_widget ()
         search_pending = true;
     });
 
-    QObject::connect (button, & QPushButton::clicked, [chooser] ()
-    {
-        char * path = g_filename_from_utf8 (chooser->text ().toUtf8 (), -1,
-         nullptr, nullptr, nullptr);
+    QObject::connect (chooser, & QLineEdit::textEdited, [button] (const QString & text)
+        { button->setDisabled (text.isEmpty ()); });
 
-        if (path && path[0])
+    auto refresh = [chooser] () {
+        QByteArray path = chooser->text ().toUtf8 ();
+        if (! path.isEmpty ())
         {
-            begin_add (path);
+            begin_add (strstr (path, "://") ? path : filename_to_uri (path));
             update_database ();
         }
+    };
 
-        g_free (path);
-    });
+    QObject::connect (chooser, & QLineEdit::returnPressed, refresh);
+    QObject::connect (button, & QPushButton::clicked, refresh);
 
     return widget;
 }
